#include "../keen.h"
#include "../keenext.h"

// AI for the scaredy cat creature (ep3)
#define SCAREDY_WALK    0
#define SCAREDY_SPOOK   1
#define SCAREDY_FLEE    2
#define SCAREDY_EXPLODE 3
#define SCAREDY_DEAD    4

#define SCAREDY_WALK_SPEED      2
#define SCAREDY_WALK_ANIM_RATE  80

#define SCAREDY_FLEE_SPEED      20
#define SCAREDY_FLEE_ANIM_RATE  16

#define SCAREDY_SPOOK_SHOW_TIME    50

#define SCAREDY_EXPLODE_ANIM_RATE  35

#define SCAREDY_SPOOK_TIME         150
#define SCAREDY_RELAX_TIME         800

#define SCAREDY_WALK_LEFT_FRAME    93
#define SCAREDY_WALK_RIGHT_FRAME   95
#define SCAREDY_SPOOK_FRAME        97
#define SCAREDY_EXPLODE_FRAME      97
#define SCAREDY_DEAD_FRAME         101

void scaredy_ai(int o)
{
int i, onsamelevel;
  if (objects[o].needinit)
  {
    objects[o].ai.scaredy.state = SCAREDY_WALK;
    objects[o].ai.scaredy.dir = RIGHT;
    objects[o].ai.scaredy.animframe = 0;
    objects[o].ai.scaredy.animtimer = 0;
    objects[o].ai.scaredy.OnSameLevelTime = 0;
    objects[o].blockedr = 0;
    objects[o].canbezapped = 1;
    objects[o].needinit = 0;
  }
  if (objects[o].ai.scaredy.state==SCAREDY_DEAD) return;

  if (objects[o].zapped || objects[o].hit)
  {
    if (objects[o].ai.scaredy.state != SCAREDY_EXPLODE)
    {
      objects[o].ai.scaredy.animframe = 0;
      objects[o].ai.scaredy.animtimer = 0;
      objects[o].ai.scaredy.state = SCAREDY_EXPLODE;
      objects[o].canbezapped = 0;
      if (objects[o].onscreen) sound_play(SOUND_YORP_DIE);
    }
  }

   // find out if a player is on the same level as the scaredy cat
   onsamelevel = 0;
   for(i=0;i<numplayers;i++)
   {
     if (player[i].isPlaying)
     {
       if ((player[i].playy >= objects[o].y-(24<<CSF)) && ((player[i].playy>>CSF)+sprites[0].ysize <= (objects[o].y>>CSF)+sprites[objects[o].sprite].ysize+24))
       {
         onsamelevel = 1;
         objects[o].ai.scaredy.SpookedByWho = i;
         break;
       }
     }
   }

  switch(objects[o].ai.scaredy.state)
  {
   case SCAREDY_WALK:
     // if player is on the same level for SCAREDY_SPOOK_TIME, run away
     if (onsamelevel)
     {
       if (objects[o].ai.scaredy.OnSameLevelTime > SCAREDY_SPOOK_TIME)
       {
         objects[o].ai.scaredy.state = SCAREDY_SPOOK;
         objects[o].ai.scaredy.spooktimer = 0;
         if (objects[o].onscreen) sound_play(SOUND_YORP_DIE);
       }
       else objects[o].ai.scaredy.OnSameLevelTime++;
     }
     else objects[o].ai.scaredy.OnSameLevelTime = 0;

     if (objects[o].ai.scaredy.dir == RIGHT)
     {  // walking right
       objects[o].sprite = SCAREDY_WALK_RIGHT_FRAME + objects[o].ai.scaredy.animframe;
       if (objects[o].blockedr)
       {
         objects[o].ai.scaredy.dir = LEFT;
       }
       else
       {
         objects[o].x += SCAREDY_WALK_SPEED;
       }
     }
     else
     {  // walking left
       objects[o].sprite = SCAREDY_WALK_LEFT_FRAME + objects[o].ai.scaredy.animframe;
       if (objects[o].blockedl)
       {
         objects[o].ai.scaredy.dir = RIGHT;
       }
       else
       {
         objects[o].x -= SCAREDY_WALK_SPEED;
       }
     }

     /* walk animation */
     if (objects[o].ai.scaredy.animtimer > SCAREDY_WALK_ANIM_RATE)
     {
       objects[o].ai.scaredy.animframe ^= 1;
       objects[o].ai.scaredy.animtimer = 0;
     }
     else objects[o].ai.scaredy.animtimer++;
   break;

   case SCAREDY_SPOOK:
      objects[o].sprite = SCAREDY_SPOOK_FRAME;

      if (objects[o].ai.scaredy.spooktimer > SCAREDY_SPOOK_SHOW_TIME)
      {
         objects[o].ai.scaredy.state = SCAREDY_FLEE;
         objects[o].ai.scaredy.OffOfSameLevelTime = 0;
         // run away from the offending player
         if (player[objects[o].ai.scaredy.SpookedByWho].playx < objects[o].x)
         {
           objects[o].ai.scaredy.dir = RIGHT;
         }
         else
         {
           objects[o].ai.scaredy.dir = LEFT;
         }
      }
      else objects[o].ai.scaredy.spooktimer++;
   break;

   case SCAREDY_FLEE:
     // if player is off of the same level for SCAREDY_RELAX_TIME,
     // we can stop fleeing
     if (!onsamelevel)
     {
       if (objects[o].ai.scaredy.OffOfSameLevelTime > SCAREDY_RELAX_TIME)
       {
         objects[o].ai.scaredy.state = SCAREDY_WALK;
         objects[o].ai.scaredy.OnSameLevelTime = 0;
       }
       else objects[o].ai.scaredy.OffOfSameLevelTime++;
     }
     else objects[o].ai.scaredy.OffOfSameLevelTime = 0;

     if (objects[o].ai.scaredy.dir == RIGHT)
     {  // walking right
       objects[o].sprite = SCAREDY_WALK_RIGHT_FRAME + objects[o].ai.scaredy.animframe;
       if (!objects[o].blockedr)
       {
         objects[o].x += SCAREDY_FLEE_SPEED;
       }
     }
     else
     {  // walking left
       objects[o].sprite = SCAREDY_WALK_LEFT_FRAME + objects[o].ai.scaredy.animframe;
       if (!objects[o].blockedl)
       {
         objects[o].x -= SCAREDY_FLEE_SPEED;
       }
     }

     /* walk animation */
     if (objects[o].ai.scaredy.animtimer > SCAREDY_FLEE_ANIM_RATE)
     {
       objects[o].ai.scaredy.animframe ^= 1;
       objects[o].ai.scaredy.animtimer = 0;
     }
     else objects[o].ai.scaredy.animtimer++;
   break;

   case SCAREDY_EXPLODE:
     // ahhhhh; I'm sorry.....you poor little thing......
     objects[o].sprite = SCAREDY_EXPLODE_FRAME + objects[o].ai.scaredy.animframe;
     if (objects[o].sprite==SCAREDY_DEAD_FRAME)
     {
        objects[o].ai.scaredy.state = SCAREDY_DEAD;
     }
     else
     {
        if (objects[o].ai.scaredy.animtimer > SCAREDY_EXPLODE_ANIM_RATE)
        {
           objects[o].ai.scaredy.animframe++;
           objects[o].ai.scaredy.animtimer = 0;
        }
        else objects[o].ai.scaredy.animtimer++;
     }
   break;
  }
}
